@@ -22,6 +22,7 @@
 #include "pcheck.h"
 #include "common.h"
 #include "stdlib.h"
+#include <string.h>
 #include <assert.h>
 
 HostType hostTypesC[] =
@@ -36,6 +37,30 @@ HostType hostTypesC[] =
 	{ "unsigned", "long",  "ulong",   false,  0,         ULONG_MAX,  sizeof(unsigned long) }
 };
 
+#define S8BIT_MIN  -128
+#define S8BIT_MAX  127
+
+#define U8BIT_MIN  0
+#define U8BIT_MAX  255
+
+#define S16BIT_MIN -32768
+#define S16BIT_MAX 32767
+
+#define U16BIT_MIN 0
+#define U16BIT_MAX 65535
+
+#define S32BIT_MIN –2147483648l
+#define S32BIT_MAX 2147483647l
+
+#define U32BIT_MIN 0
+#define U32BIT_MAX 4294967295l
+
+#define S64BIT_MIN –9223372036854775808ll
+#define S64BIT_MAX 9223372036854775807ll
+
+#define U64BIT_MIN 0
+#define U64BIT_MAX 18446744073709551615ll
+
 HostType hostTypesD[] =
 {
 	{ "byte",    0,  "byte",    true,   CHAR_MIN,  CHAR_MAX,    1 },
@@ -64,10 +89,24 @@ HostType hostTypesRuby[] =
 	{ "int",     0,  "int",    true,   INT_MIN,   INT_MAX,     4 },
 };
 
+HostType hostTypesCSharp[] =
+{
+	{ "sbyte",    0, "sbyte",   true,   CHAR_MIN,  CHAR_MAX,    1 },
+	{ "byte",   0,   "byte",    false,  0,         UCHAR_MAX,   1 },
+	{ "short",   0,  "short",   true,   SHRT_MIN,  SHRT_MAX,    2 },
+	{ "ushort",  0,  "ushort",  false,  0,         USHRT_MAX,   2 },
+	{ "char",    0,  "char",    false,  0,         USHRT_MAX,   2 },
+	{ "int",     0,  "int",     true,   INT_MIN,   INT_MAX,     4 },
+	{ "uint",    0,  "uint",    false,  0,         UINT_MAX,    4 },
+	{ "long",    0,  "long",    true,   LONG_MIN,  LONG_MAX,    8 },
+	{ "ulong",   0,  "ulong",   false,  0,         ULONG_MAX,   8 }
+};
+
 HostLang hostLangC =    { HostLang::C,    hostTypesC,    8, hostTypesC+0,    true };
 HostLang hostLangD =    { HostLang::D,    hostTypesD,    9, hostTypesD+2,    true };
 HostLang hostLangJava = { HostLang::Java, hostTypesJava, 4, hostTypesJava+2, false };
 HostLang hostLangRuby = { HostLang::Ruby, hostTypesRuby, 2, hostTypesRuby+0, false };
+HostLang hostLangCSharp = { HostLang::CSharp, hostTypesCSharp, 9, hostTypesCSharp+4, true };
 
 HostLang *hostLang = &hostLangC;
 
@@ -109,7 +148,7 @@ HostType *findAlphTypeInternal( char *s1 )
 }
 
 /* Construct a new parameter checker with for paramSpec. */
-ParamCheck::ParamCheck(char *paramSpec, int argc, char **argv)
+ParamCheck::ParamCheck( const char *paramSpec, int argc, char **argv )
 :
 	state(noparam),
 	argOffset(0),
@@ -152,7 +191,7 @@ bool ParamCheck::check()
 			 argv[iCurArg][0] != '-' ||   /* Not a param. */
 			 argv[iCurArg][1] == 0 ) {    /* Only a dash. */
 			parameter = 0;
-			parameterArg = 0;
+			paramArg = 0;
 
 			iCurArg += 1;
 			state = noparam;
@@ -165,7 +204,7 @@ bool ParamCheck::check()
 	char argChar = *argOffset;
 	
 	/* Loop over all the parms and look for a match. */
-	char *pSpec = paramSpec;
+	const char *pSpec = paramSpec;
 	while ( *pSpec != 0 ) {
 		char pSpecChar = *pSpec;
 
@@ -191,7 +230,7 @@ bool ParamCheck::check()
 						/* We are the last arg so there
 						 * cannot be a parameter to it. */
 						parameter = argChar;
-						parameterArg = 0;
+						paramArg = 0;
 						iCurArg += 1;
 						argOffset = 0;
 						state = invalid;
@@ -200,7 +239,7 @@ bool ParamCheck::check()
 					else {
 						/* the parameter to the arg is the next arg. */
 						parameter = pSpecChar;
-						parameterArg = argv[iCurArg + 1];
+						paramArg = argv[iCurArg + 1];
 						iCurArg += 2;
 						argOffset = 0;
 						state = match;
@@ -210,7 +249,7 @@ bool ParamCheck::check()
 				else {
 					/* The param for the arg is built in. */
 					parameter = pSpecChar;
-					parameterArg = argOffset + 1;
+					paramArg = argOffset + 1;
 					iCurArg += 1;
 					argOffset = 0;
 					state = match;
@@ -221,7 +260,7 @@ bool ParamCheck::check()
 				/* Good, we matched the parm and no
 				 * arg is required. */
 				parameter = pSpecChar;
-				parameterArg = 0;
+				paramArg = 0;
 				argOffset += 1;
 				state = match;
 				return true;
@@ -231,7 +270,7 @@ bool ParamCheck::check()
 
 	/* We did not find a match. Bad Argument. */
 	parameter = argChar;
-	parameterArg = 0;
+	paramArg = 0;
 	argOffset += 1;
 	state = invalid;
 	return true;
@@ -287,7 +326,7 @@ char *findFileExtension( char *stemFile )
 
 /* Make a file name from a stem. Removes the old filename suffix and
  * replaces it with a new one. Returns a newed up string. */
-char *fileNameFromStem( char *stemFile, char *suffix )
+char *fileNameFromStem( char *stemFile, const char *suffix )
 {
 	long len = strlen( stemFile );
 	assert( len > 0 );
