@@ -198,11 +198,13 @@ struct Token
 	int length;
 	InputLoc loc;
 
-	void prepareLitString( Token &result, bool &caseInsensitive );
 	void append( const Token &other );
-	void set( char *str, int len );
+	void set( const char *str, int len );
 };
 
+char *prepareLitString( const InputLoc &loc, char *src, long length, 
+			long &resLen, bool &caseInsensitive );
+
 /* Store the value and type of a priority augmentation. */
 struct PriorityAug
 {
@@ -219,7 +221,7 @@ struct PriorityAug
  */
 struct VarDef
 {
-	VarDef( char *name, JoinOrLm *joinOrLm )
+	VarDef( const char *name, JoinOrLm *joinOrLm )
 		: name(name), joinOrLm(joinOrLm), isExport(false) { }
 	
 	/* Parse tree traversal. */
@@ -227,7 +229,7 @@ struct VarDef
 	void makeNameTree( const InputLoc &loc, ParseData *pd );
 	void resolveNameRefs( ParseData *pd );
 
-	char *name;
+	const char *name;
 	JoinOrLm *joinOrLm;
 	bool isExport;
 };
@@ -298,8 +300,9 @@ struct LongestMatch
 	FsmAp *walk( ParseData *pd );
 	void makeNameTree( ParseData *pd );
 	void resolveNameRefs( ParseData *pd );
-	void runLonestMatch( ParseData *pd, FsmAp *graph );
-	Action *newAction( ParseData *pd, const InputLoc &loc, char *name, 
+	void transferScannerLeavingActions( FsmAp *graph );
+	void runLongestMatch( ParseData *pd, FsmAp *graph );
+	Action *newAction( ParseData *pd, const InputLoc &loc, const char *name, 
 			InlineList *inlineList );
 	void makeActions( ParseData *pd );
 	void findName( ParseData *pd );
@@ -307,7 +310,7 @@ struct LongestMatch
 
 	InputLoc loc;
 	LmPartList *longestMatchList;
-	char *name;
+	const char *name;
 
 	Action *lmActSelect;
 	bool lmSwitchHandlesError;
@@ -327,9 +330,9 @@ struct JoinOrLm
 	};
 
 	JoinOrLm( Join *join ) : 
-		join(join), type(JoinType) {}
+		join(join), longestMatch(0), type(JoinType) {}
 	JoinOrLm( LongestMatch *longestMatch ) :
-		longestMatch(longestMatch), type(LongestMatchType) {}
+		join(0), longestMatch(longestMatch), type(LongestMatchType) {}
 
 	FsmAp *walk( ParseData *pd );
 	void makeNameTree( ParseData *pd );
