@@ -68,8 +68,8 @@ struct Scanner
 #line 69 "xmlscan.cpp"
 	{
 	cs = Scanner_start;
-	tokstart = 0;
-	tokend = 0;
+	ts = 0;
+	te = 0;
 	act = 0;
 	}
 #line 63 "xmlscan.rl"
@@ -85,7 +85,7 @@ struct Scanner
 
 	/* Scanner State. */
 	int cs, act, have, curline, curcol;
-	char *tokstart, *tokend;
+	char *ts, *te;
 	char *p, *pe;
 	int done;
 
@@ -110,7 +110,7 @@ struct Scanner
 #define TK_OpenTag 4
 #define TK_CloseTag 5
 
-#define ret_tok( _tok ) token = (_tok); data = tokstart
+#define ret_tok( _tok ) token = (_tok); data = ts
 
 void Scanner::adjustAttrPointers( int distance )
 {
@@ -141,21 +141,21 @@ int Scanner::scan( )
 		if ( p == pe ) {
 			//printf("scanner: need more data\n");
 
-			if ( tokstart == 0 )
+			if ( ts == 0 )
 				have = 0;
 			else {
 				/* There is data that needs to be shifted over. */
 				//printf("scanner: buffer broken mid token\n");
-				have = pe - tokstart;
-				memmove( buf, tokstart, have );
+				have = pe - ts;
+				memmove( buf, ts, have );
 
-				int distance = tokstart - buf;
-				tokend -= distance;
+				int distance = ts - buf;
+				te -= distance;
 				tag_id_start -= distance;
 				attr_id_start -= distance;
 				attr_value_start -= distance;
 				adjustAttrPointers( distance );
-				tokstart = buf;
+				ts = buf;
 			}
 
 			p = buf + have;
@@ -187,26 +187,26 @@ int Scanner::scan( )
 #line 188 "xmlscan.cpp"
 	{
 	if ( p == pe )
-		goto _out;
+		goto _test_eof;
 	switch ( cs )
 	{
 tr6:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 168 "xmlscan.rl"
-	{tokend = p+1;{ buffer.append( '&' ); }}
+	{te = p+1;{ buffer.append( '&' ); }}
 	goto st20;
 tr8:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 172 "xmlscan.rl"
-	{tokend = p+1;{ buffer.append( '>' ); }}
+	{te = p+1;{ buffer.append( '>' ); }}
 	goto st20;
 tr10:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 170 "xmlscan.rl"
-	{tokend = p+1;{ buffer.append( '<' ); }}
+	{te = p+1;{ buffer.append( '<' ); }}
 	goto st20;
 tr20:
 #line 150 "xmlscan.rl"
@@ -214,13 +214,13 @@ tr20:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 160 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_CloseTag ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_CloseTag ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr23:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 160 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_CloseTag ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_CloseTag ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr27:
 #line 150 "xmlscan.rl"
@@ -228,13 +228,13 @@ tr27:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 157 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_OpenTag ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_OpenTag ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr30:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 157 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_OpenTag ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_OpenTag ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr46:
 #line 132 "xmlscan.rl"
@@ -251,19 +251,19 @@ tr46:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 157 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_OpenTag ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_OpenTag ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr48:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 164 "xmlscan.rl"
-	{tokend = p+1;{ buffer.append( *p ); }}
+	{te = p+1;{ buffer.append( *p ); }}
 	goto st20;
 tr49:
 #line 116 "xmlscan.rl"
 	{ token_col = curcol; token_line = curline; }
 #line 175 "xmlscan.rl"
-	{tokend = p+1;{ ret_tok( TK_EOF ); goto _out20; }}
+	{te = p+1;{ ret_tok( TK_EOF ); {p++; cs = 20; goto _out;} }}
 	goto st20;
 tr50:
 #line 117 "xmlscan.rl"
@@ -271,16 +271,16 @@ tr50:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 #line 164 "xmlscan.rl"
-	{tokend = p+1;{ buffer.append( *p ); }}
+	{te = p+1;{ buffer.append( *p ); }}
 	goto st20;
 st20:
 #line 1 "xmlscan.rl"
-	{tokstart = 0;}
+	{ts = 0;}
 	if ( ++p == pe )
-		goto _out20;
+		goto _test_eof20;
 case 20:
 #line 1 "xmlscan.rl"
-	{tokstart = p;}
+	{ts = p;}
 #line 285 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 0: goto tr49;
@@ -295,7 +295,7 @@ tr51:
 	goto st1;
 st1:
 	if ( ++p == pe )
-		goto _out1;
+		goto _test_eof1;
 case 1:
 #line 301 "xmlscan.cpp"
 	switch( (*p) ) {
@@ -305,16 +305,17 @@ case 1:
 	}
 	goto st0;
 st0:
-	goto _out0;
+cs = 0;
+	goto _out;
 tr0:
 #line 115 "xmlscan.rl"
 	{ curcol++; }
 	goto st2;
 st2:
 	if ( ++p == pe )
-		goto _out2;
+		goto _test_eof2;
 case 2:
-#line 318 "xmlscan.cpp"
+#line 319 "xmlscan.cpp"
 	if ( (*p) == 109 )
 		goto tr4;
 	goto st0;
@@ -324,9 +325,9 @@ tr4:
 	goto st3;
 st3:
 	if ( ++p == pe )
-		goto _out3;
+		goto _test_eof3;
 case 3:
-#line 330 "xmlscan.cpp"
+#line 331 "xmlscan.cpp"
 	if ( (*p) == 112 )
 		goto tr5;
 	goto st0;
@@ -336,9 +337,9 @@ tr5:
 	goto st4;
 st4:
 	if ( ++p == pe )
-		goto _out4;
+		goto _test_eof4;
 case 4:
-#line 342 "xmlscan.cpp"
+#line 343 "xmlscan.cpp"
 	if ( (*p) == 59 )
 		goto tr6;
 	goto st0;
@@ -348,9 +349,9 @@ tr2:
 	goto st5;
 st5:
 	if ( ++p == pe )
-		goto _out5;
+		goto _test_eof5;
 case 5:
-#line 354 "xmlscan.cpp"
+#line 355 "xmlscan.cpp"
 	if ( (*p) == 116 )
 		goto tr7;
 	goto st0;
@@ -360,9 +361,9 @@ tr7:
 	goto st6;
 st6:
 	if ( ++p == pe )
-		goto _out6;
+		goto _test_eof6;
 case 6:
-#line 366 "xmlscan.cpp"
+#line 367 "xmlscan.cpp"
 	if ( (*p) == 59 )
 		goto tr8;
 	goto st0;
@@ -372,9 +373,9 @@ tr3:
 	goto st7;
 st7:
 	if ( ++p == pe )
-		goto _out7;
+		goto _test_eof7;
 case 7:
-#line 378 "xmlscan.cpp"
+#line 379 "xmlscan.cpp"
 	if ( (*p) == 116 )
 		goto tr9;
 	goto st0;
@@ -384,9 +385,9 @@ tr9:
 	goto st8;
 st8:
 	if ( ++p == pe )
-		goto _out8;
+		goto _test_eof8;
 case 8:
-#line 390 "xmlscan.cpp"
+#line 391 "xmlscan.cpp"
 	if ( (*p) == 59 )
 		goto tr10;
 	goto st0;
@@ -408,9 +409,9 @@ tr52:
 	goto st9;
 st9:
 	if ( ++p == pe )
-		goto _out9;
+		goto _test_eof9;
 case 9:
-#line 414 "xmlscan.cpp"
+#line 415 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr11;
 		case 10: goto tr12;
@@ -437,9 +438,9 @@ tr15:
 	goto st10;
 st10:
 	if ( ++p == pe )
-		goto _out10;
+		goto _test_eof10;
 case 10:
-#line 443 "xmlscan.cpp"
+#line 444 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr13;
 		case 10: goto tr15;
@@ -465,9 +466,9 @@ tr16:
 	goto st11;
 st11:
 	if ( ++p == pe )
-		goto _out11;
+		goto _test_eof11;
 case 11:
-#line 471 "xmlscan.cpp"
+#line 472 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr17;
 		case 10: goto tr18;
@@ -511,9 +512,9 @@ tr18:
 	goto st12;
 st12:
 	if ( ++p == pe )
-		goto _out12;
+		goto _test_eof12;
 case 12:
-#line 517 "xmlscan.cpp"
+#line 518 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr21;
 		case 10: goto tr22;
@@ -534,9 +535,9 @@ tr14:
 	goto st13;
 st13:
 	if ( ++p == pe )
-		goto _out13;
+		goto _test_eof13;
 case 13:
-#line 540 "xmlscan.cpp"
+#line 541 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr24;
 		case 10: goto tr25;
@@ -612,9 +613,9 @@ tr45:
 	goto st14;
 st14:
 	if ( ++p == pe )
-		goto _out14;
+		goto _test_eof14;
 case 14:
-#line 618 "xmlscan.cpp"
+#line 619 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr28;
 		case 10: goto tr29;
@@ -658,9 +659,9 @@ tr47:
 	goto st15;
 st15:
 	if ( ++p == pe )
-		goto _out15;
+		goto _test_eof15;
 case 15:
-#line 664 "xmlscan.cpp"
+#line 665 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr32;
 		case 10: goto tr33;
@@ -704,9 +705,9 @@ tr33:
 	goto st16;
 st16:
 	if ( ++p == pe )
-		goto _out16;
+		goto _test_eof16;
 case 16:
-#line 710 "xmlscan.cpp"
+#line 711 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr36;
 		case 10: goto tr37;
@@ -733,9 +734,9 @@ tr35:
 	goto st17;
 st17:
 	if ( ++p == pe )
-		goto _out17;
+		goto _test_eof17;
 case 17:
-#line 739 "xmlscan.cpp"
+#line 740 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr38;
 		case 10: goto tr39;
@@ -762,9 +763,9 @@ tr40:
 	goto st18;
 st18:
 	if ( ++p == pe )
-		goto _out18;
+		goto _test_eof18;
 case 18:
-#line 768 "xmlscan.cpp"
+#line 769 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr42;
 		case 34: goto tr43;
@@ -776,9 +777,9 @@ tr43:
 	goto st19;
 st19:
 	if ( ++p == pe )
-		goto _out19;
+		goto _test_eof19;
 case 19:
-#line 782 "xmlscan.cpp"
+#line 783 "xmlscan.cpp"
 	switch( (*p) ) {
 		case 9: goto tr44;
 		case 10: goto tr45;
@@ -794,28 +795,28 @@ case 19:
 		goto tr47;
 	goto st0;
 	}
-	_out20: cs = 20; goto _out; 
-	_out1: cs = 1; goto _out; 
-	_out0: cs = 0; goto _out; 
-	_out2: cs = 2; goto _out; 
-	_out3: cs = 3; goto _out; 
-	_out4: cs = 4; goto _out; 
-	_out5: cs = 5; goto _out; 
-	_out6: cs = 6; goto _out; 
-	_out7: cs = 7; goto _out; 
-	_out8: cs = 8; goto _out; 
-	_out9: cs = 9; goto _out; 
-	_out10: cs = 10; goto _out; 
-	_out11: cs = 11; goto _out; 
-	_out12: cs = 12; goto _out; 
-	_out13: cs = 13; goto _out; 
-	_out14: cs = 14; goto _out; 
-	_out15: cs = 15; goto _out; 
-	_out16: cs = 16; goto _out; 
-	_out17: cs = 17; goto _out; 
-	_out18: cs = 18; goto _out; 
-	_out19: cs = 19; goto _out; 
+	_test_eof20: cs = 20; goto _test_eof; 
+	_test_eof1: cs = 1; goto _test_eof; 
+	_test_eof2: cs = 2; goto _test_eof; 
+	_test_eof3: cs = 3; goto _test_eof; 
+	_test_eof4: cs = 4; goto _test_eof; 
+	_test_eof5: cs = 5; goto _test_eof; 
+	_test_eof6: cs = 6; goto _test_eof; 
+	_test_eof7: cs = 7; goto _test_eof; 
+	_test_eof8: cs = 8; goto _test_eof; 
+	_test_eof9: cs = 9; goto _test_eof; 
+	_test_eof10: cs = 10; goto _test_eof; 
+	_test_eof11: cs = 11; goto _test_eof; 
+	_test_eof12: cs = 12; goto _test_eof; 
+	_test_eof13: cs = 13; goto _test_eof; 
+	_test_eof14: cs = 14; goto _test_eof; 
+	_test_eof15: cs = 15; goto _test_eof; 
+	_test_eof16: cs = 16; goto _test_eof; 
+	_test_eof17: cs = 17; goto _test_eof; 
+	_test_eof18: cs = 18; goto _test_eof; 
+	_test_eof19: cs = 19; goto _test_eof; 
 
+	_test_eof: {}
 	_out: {}
 	}
 #line 239 "xmlscan.rl"
@@ -824,8 +825,6 @@ case 19:
 			return TK_ERR;
 
 		if ( token != TK_NO_TOKEN ) {
-			/* fbreak does not advance p, so we do it manually. */
-			p = p + 1;
 			data_len = p - data;
 			return token;
 		}
@@ -898,7 +897,7 @@ int xml_parse( std::istream &input, char *fileName,
 					case TAG_set_act: case TAG_start_state:
 					case TAG_error_state: case TAG_state_actions: 
 					case TAG_action_table: case TAG_cond_space: 
-					case TAG_c: case TAG_ex:
+					case TAG_c: case TAG_ex: case TAG_eof_t:
 						tag->content = new char[scanner.buffer.length+1];
 						memcpy( tag->content, scanner.buffer.data,
 								scanner.buffer.length );
