@@ -71,7 +71,7 @@ std::ostream &FlatCodeGen::TO_STATE_ACTION_SWITCH()
 		if ( act->numToStateRefs > 0 ) {
 			/* Write the case label, the action and the case break */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -88,7 +88,7 @@ std::ostream &FlatCodeGen::FROM_STATE_ACTION_SWITCH()
 		if ( act->numFromStateRefs > 0 ) {
 			/* Write the case label, the action and the case break */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -105,7 +105,7 @@ std::ostream &FlatCodeGen::EOF_ACTION_SWITCH()
 		if ( act->numEofRefs > 0 ) {
 			/* Write the case label, the action and the case break */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, true );
+			ACTION( out, act, 0, true, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -123,7 +123,7 @@ std::ostream &FlatCodeGen::ACTION_SWITCH()
 		if ( act->numTransRefs > 0 ) {
 			/* Write the case label, the action and the case break */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -228,6 +228,31 @@ std::ostream &FlatCodeGen::EOF_ACTIONS()
 	return out;
 }
 
+std::ostream &FlatCodeGen::EOF_TRANS()
+{
+	out << "\t";
+	int totalStateNum = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write any eof action. */
+
+		long trans = 0;
+		if ( st->eofTrans != 0 ) {
+			assert( st->eofTrans->pos >= 0 );
+			trans = st->eofTrans->pos+1;
+		}
+		out << trans;
+
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+	return out;
+}
+
+
 std::ostream &FlatCodeGen::COND_KEYS()
 {
 	out << '\t';
@@ -371,8 +396,11 @@ std::ostream &FlatCodeGen::TRANS_TARGS()
 	out << '\t';
 	int totalStates = 0;
 	for ( int t = 0; t < redFsm->transSet.length(); t++ ) {
-		/* Write out the target state. */
+		/* Save the position. Needed for eofTargs. */
 		RedTransAp *trans = transPtrs[t];
+		trans->pos = t;
+
+		/* Write out the target state. */
 		out << trans->targ->id;
 		if ( t < redFsm->transSet.length()-1 ) {
 			out << ", ";
@@ -433,7 +461,7 @@ void FlatCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 void FlatCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
@@ -455,35 +483,58 @@ void FlatCodeGen::NEXT( ostream &ret, int nextDest, bool inFinish )
 void FlatCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << ");";
 }
 
 void FlatCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
 			callDest << "; " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 
 void FlatCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, targState, inFinish );
+	INLINE_LIST( ret, ilItem->children, targState, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 
 void FlatCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; " << 
-			CTRL_FLOW() << "goto _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "];";
+
+	if ( postPopExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, postPopExpr, 0, false, false );
+		ret << "}";
+	}
+
+	ret << CTRL_FLOW() << "goto _again;}";
 }
 
-void FlatCodeGen::BREAK( ostream &ret, int targState )
+void FlatCodeGen::BREAK( ostream &ret, int targState, bool csForced )
 {
 	outLabelUsed = true;
-	ret << CTRL_FLOW() << "goto _out;";
+	ret << "{" << P() << "++; " << CTRL_FLOW() << "goto _out; }";
 }
 
 void FlatCodeGen::writeData()
@@ -572,6 +623,13 @@ void FlatCodeGen::writeData()
 		"\n";
 	}
 
+	if ( redFsm->anyEofTrans() ) {
+		OPEN_ARRAY( ARRAY_TYPE(redFsm->maxIndexOffset+1), ET() );
+		EOF_TRANS();
+		CLOSE_ARRAY() <<
+		"\n";
+	}
+
 	STATE_IDS();
 }
 
@@ -618,6 +676,7 @@ void FlatCodeGen::COND_TRANSLATE()
 
 void FlatCodeGen::writeExec()
 {
+	testEofUsed = false;
 	outLabelUsed = false;
 
 	out << 
@@ -656,10 +715,10 @@ void FlatCodeGen::writeExec()
 	out << "\n";
 
 	if ( hasEnd ) {
-		outLabelUsed = true;
+		testEofUsed = true;
 		out << 
 			"	if ( " << P() << " == " << PE() << " )\n"
-			"		goto _out;\n";
+			"		goto _test_eof;\n";
 	}
 
 	if ( redFsm->errState != 0 ) {
@@ -689,6 +748,9 @@ void FlatCodeGen::writeExec()
 
 	LOCATE_TRANS();
 
+	if ( redFsm->anyEofTrans() )
+		out << "_eof_trans:\n";
+
 	if ( redFsm->anyRegCurStateRef() )
 		out << "	_ps = " << CS() << ";\n";
 
@@ -747,27 +809,43 @@ void FlatCodeGen::writeExec()
 			"	goto _resume;\n";
 	}
 
+	if ( testEofUsed )
+		out << "	_test_eof: {}\n";
+
+	if ( redFsm->anyEofTrans() || redFsm->anyEofActions() ) {
+		out << 
+			"	if ( " << P() << " == " << EOFV() << " )\n"
+			"	{\n";
+
+		if ( redFsm->anyEofTrans() ) {
+			out <<
+				"	if ( " << ET() << "[" << CS() << "] > 0 ) {\n"
+				"		_trans = " << ET() << "[" << CS() << "] - 1;\n"
+				"		goto _eof_trans;\n"
+				"	}\n";
+		}
+
+		if ( redFsm->anyEofActions() ) {
+			out <<
+				"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << 
+						POINTER() << "__acts = " << 
+						ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
+				"	" << UINT() << " __nacts = " << CAST(UINT()) << " *__acts++;\n"
+				"	while ( __nacts-- > 0 ) {\n"
+				"		switch ( *__acts++ ) {\n";
+				EOF_ACTION_SWITCH();
+				SWITCH_DEFAULT() <<
+				"		}\n"
+				"	}\n";
+		}
+
+		out <<
+			"	}\n"
+			"\n";
+	}
+
 	if ( outLabelUsed )
 		out << "	_out: {}\n";
 
 	out << "	}\n";
 }
-
-void FlatCodeGen::writeEOF()
-{
-	if ( redFsm->anyEofActions() ) {
-		out << 
-			"	{\n"
-			"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << POINTER() << "_acts = " << 
-					ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
-			"	" << UINT() << " _nacts = " << CAST(UINT()) << " *_acts++;\n"
-			"	while ( _nacts-- > 0 ) {\n"
-			"		switch ( *_acts++ ) {\n";
-			EOF_ACTION_SWITCH();
-			SWITCH_DEFAULT() <<
-			"		}\n"
-			"	}\n"
-			"	}\n"
-			"\n";
-	}
-}
