@@ -28,17 +28,23 @@
 #include <sstream>
 
 /* Integer array line length. */
-#define IALL 8
+#define IALL 12
 
 /* Static array initialization item count 
  * (should be multiple of IALL). */
-#define SAIIC 8192
+#define SAIIC 8184
 
-using std::ostream;
+#define _resume    1
+#define _again     2
+#define _eof_trans 3
+#define _test_eof  4
+#define _out       5
+
+using std::setw;
+using std::ios;
 using std::ostringstream;
 using std::string;
 using std::cerr;
-using std::endl;
 
 void lineDirective( ostream &out, char *fileName, int line )
 {
@@ -62,39 +68,63 @@ void genLineDirective( ostream &out )
 
 void JavaTabCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << "{" << CS() << " = " << gotoDest << "; " << 
-			CTRL_FLOW() << "break _again;}";
+	ret << "{" << CS() << " = " << gotoDest << "; _goto_targ = " << _again << "; " << 
+			CTRL_FLOW() << "continue _goto;}";
 }
 
 void JavaTabCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, 0, inFinish );
-	ret << "); " << CTRL_FLOW() << "break _again;}";
+	ret << "); _goto_targ = " << _again << "; " << CTRL_FLOW() << "continue _goto;}";
 }
 
 void JavaTabCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
-			callDest << "; " << CTRL_FLOW() << "break _again;}";
+			callDest << "; _goto_targ = " << _again << "; " << CTRL_FLOW() << "continue _goto;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void JavaTabCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, targState, inFinish );
-	ret << "); " << CTRL_FLOW() << "break _again;}";
+	ret << "); _goto_targ = " << _again << "; " << CTRL_FLOW() << "continue _goto;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void JavaTabCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() 
-			<< "]; " << CTRL_FLOW() << "break _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "];";
+
+	if ( postPopExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, postPopExpr, 0, false );
+		ret << "}";
+	}
+
+	ret << "_goto_targ = " << _again << "; " << CTRL_FLOW() << "continue _goto;}";
 }
 
 void JavaTabCodeGen::BREAK( ostream &ret, int targState )
 {
-	ret << CTRL_FLOW() << "break _resume;";
+	ret << "{ " << P() << " += 1; _goto_targ = " << _out << "; " << 
+			CTRL_FLOW() << " continue _goto;}";
 }
 
 void JavaTabCodeGen::NEXT( ostream &ret, int nextDest, bool inFinish )
@@ -576,6 +606,23 @@ std::ostream &JavaTabCodeGen::EOF_ACTIONS()
 	return out;
 }
 
+std::ostream &JavaTabCodeGen::EOF_TRANS()
+{
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write any eof action. */
+		long trans = 0;
+		if ( st->eofTrans != 0 ) {
+			assert( st->eofTrans->pos >= 0 );
+			trans = st->eofTrans->pos+1;
+		}
+
+		/* Write any eof action. */
+		ARRAY_ITEM( INT(trans), st.last() );
+	}
+	return out;
+}
+
+
 std::ostream &JavaTabCodeGen::COND_KEYS()
 {
 	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
@@ -660,23 +707,35 @@ std::ostream &JavaTabCodeGen::INDICIES()
 
 std::ostream &JavaTabCodeGen::TRANS_TARGS()
 {
+	int totalTrans = 0;
 	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
 		/* Walk the singles. */
 		for ( RedTransList::Iter stel = st->outSingle; stel.lte(); stel++ ) {
 			RedTransAp *trans = stel->value;
 			ARRAY_ITEM( KEY( trans->targ->id ), false );
+			totalTrans++;
 		}
 
 		/* Walk the ranges. */
 		for ( RedTransList::Iter rtel = st->outRange; rtel.lte(); rtel++ ) {
 			RedTransAp *trans = rtel->value;
 			ARRAY_ITEM( KEY( trans->targ->id ), false );
+			totalTrans++;
 		}
 
 		/* The state's default target state. */
 		if ( st->defTrans != 0 ) {
 			RedTransAp *trans = st->defTrans;
 			ARRAY_ITEM( KEY( trans->targ->id ), false );
+			totalTrans++;
+		}
+	}
+
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->eofTrans != 0 ) {
+			RedTransAp *trans = st->eofTrans;
+			trans->pos = totalTrans++;
+			ARRAY_ITEM( KEY( trans->targ->id ), false );
 		}
 	}
 
@@ -709,6 +768,13 @@ std::ostream &JavaTabCodeGen::TRANS_ACTIONS()
 		}
 	}
 
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->eofTrans != 0 ) {
+			RedTransAp *trans = st->eofTrans;
+			ARRAY_ITEM( INT(TRANS_ACTION( trans )), false );
+		}
+	}
+
 	/* Output one last number so we don't have to figure out when the last
 	 * entry is and avoid writing a comma. */
 	ARRAY_ITEM( INT(0), true );
@@ -724,8 +790,11 @@ std::ostream &JavaTabCodeGen::TRANS_TARGS_WI()
 
 	/* Keep a count of the num of items in the array written. */
 	for ( int t = 0; t < redFsm->transSet.length(); t++ ) {
-		/* Write out the target state. */
+		/* Save the position. Needed for eofTargs. */
 		RedTransAp *trans = transPtrs[t];
+		trans->pos = t;
+
+		/* Write out the target state. */
 		ARRAY_ITEM( INT(trans->targ->id), ( t >= redFsm->transSet.length()-1 ) );
 	}
 	delete[] transPtrs;
@@ -872,6 +941,13 @@ void JavaTabCodeGen::writeData()
 		"\n";
 	}
 
+	if ( redFsm->anyEofTrans() ) {
+		OPEN_ARRAY( ARRAY_TYPE(redFsm->maxIndexOffset+1), ET() );
+		EOF_TRANS();
+		CLOSE_ARRAY() <<
+		"\n";
+	}
+
 	if ( redFsm->startState != 0 )
 		STATIC_VAR( "int", START() ) << " = " << START_STATE_ID() << ";\n";
 
@@ -903,7 +979,7 @@ void JavaTabCodeGen::writeExec()
 
 	out << 
 		";\n"
-		"	int _trans;\n";
+		"	int _trans = 0;\n";
 
 	if ( redFsm->anyConditions() )
 		out << "	int _widec;\n";
@@ -918,17 +994,31 @@ void JavaTabCodeGen::writeExec()
 
 	out <<
 		"	int _keys;\n"
+		"	int _goto_targ = 0;\n"
 		"\n";
+	
+	out <<
+		"	_goto: while (true) {\n"
+		"	switch ( _goto_targ ) {\n"
+		"	case 0:\n";
 
-	if ( hasEnd )
-		out << "	if ( " << P() << " != " << PE() << " ) {\n";
+	if ( hasEnd ) {
+		out << 
+			"	if ( " << P() << " == " << PE() << " ) {\n"
+			"		_goto_targ = " << _test_eof << ";\n"
+			"		continue _goto;\n"
+			"	}\n";
+	}
 
-	if ( redFsm->errState != 0 )
-		out << "	if ( " << CS() << " != " << redFsm->errState->id << " ) {\n";
+	if ( redFsm->errState != 0 ) {
+		out << 
+			"	if ( " << CS() << " == " << redFsm->errState->id << " ) {\n"
+			"		_goto_targ = " << _out << ";\n"
+			"		continue _goto;\n"
+			"	}\n";
+	}
 
-	out << "	_resume: while ( true ) {\n";
-
-	out << "	_again: do {\n";
+	out << "case " << _resume << ":\n"; 
 
 	if ( redFsm->anyFromStateActions() ) {
 		out <<
@@ -947,11 +1037,14 @@ void JavaTabCodeGen::writeExec()
 
 	LOCATE_TRANS();
 
-	if ( redFsm->anyRegCurStateRef() )
-		out << "	_ps = " << CS() << ";\n";
-
 	if ( useIndicies )
 		out << "	_trans = " << I() << "[_trans];\n";
+	
+	if ( redFsm->anyEofTrans() )
+		out << "case " << _eof_trans << ":\n";
+
+	if ( redFsm->anyRegCurStateRef() )
+		out << "	_ps = " << CS() << ";\n";
 
 	out <<
 		"	" << CS() << " = " << TT() << "[_trans];\n"
@@ -959,22 +1052,20 @@ void JavaTabCodeGen::writeExec()
 
 	if ( redFsm->anyRegActions() ) {
 		out <<
-			"	if ( " << TA() << "[_trans] == 0 )\n"
-			"		break _again;\n"
-			"\n"
-			"	_acts = " <<  TA() << "[_trans]" << ";\n"
-			"	_nacts = " << CAST("int") << " " <<  A() << "[_acts++];\n"
-			"	while ( _nacts-- > 0 )\n	{\n"
-			"		switch ( " << A() << "[_acts++] )\n"
-			"		{\n";
+			"	if ( " << TA() << "[_trans] != 0 ) {\n"
+			"		_acts = " <<  TA() << "[_trans]" << ";\n"
+			"		_nacts = " << CAST("int") << " " <<  A() << "[_acts++];\n"
+			"		while ( _nacts-- > 0 )\n	{\n"
+			"			switch ( " << A() << "[_acts++] )\n"
+			"			{\n";
 			ACTION_SWITCH() <<
+			"			}\n"
 			"		}\n"
 			"	}\n"
 			"\n";
 	}
 
-	/* Again loop, functions as again label. */
-	out << "	} while (false);\n";
+	out << "case " << _again << ":\n";
 
 	if ( redFsm->anyToStateActions() ) {
 		out <<
@@ -990,48 +1081,66 @@ void JavaTabCodeGen::writeExec()
 
 	if ( redFsm->errState != 0 ) {
 		out << 
-			"	if ( " << CS() << " == " << redFsm->errState->id << " )\n"
-			"		break _resume;\n";
+			"	if ( " << CS() << " == " << redFsm->errState->id << " ) {\n"
+			"		_goto_targ = " << _out << ";\n"
+			"		continue _goto;\n"
+			"	}\n";
 	}
 
 	if ( hasEnd ) {
 		out << 
-			"	if ( ++" << P() << " == " << PE() << " )\n"
-			"		break _resume;\n";
+			"	if ( ++" << P() << " != " << PE() << " ) {\n"
+			"		_goto_targ = " << _resume << ";\n"
+			"		continue _goto;\n"
+			"	}\n";
 	}
 	else {
 		out << 
-			"	" << P() << " += 1;\n";
+			"	" << P() << " += 1;\n"
+			"	_goto_targ = " << _resume << ";\n"
+			"	continue _goto;\n";
 	}
 
-	/* Close the resume loop. */
-	out << "	}\n";
+	out << "case " << _test_eof << ":\n"; 
+
+	if ( redFsm->anyEofTrans() || redFsm->anyEofActions() ) {
+		out <<
+			"	if ( " << P() << " == " << EOFV() << " )\n"
+			"	{\n";
+
+		if ( redFsm->anyEofTrans() ) {
+			out <<
+				"	if ( " << ET() << "[" << CS() << "] > 0 ) {\n"
+				"		_trans = " << ET() << "[" << CS() << "] - 1;\n"
+				"		_goto_targ = " << _eof_trans << ";\n"
+				"		continue _goto;\n"
+				"	}\n";
+		}
+
+		if ( redFsm->anyEofActions() ) {
+			out <<
+				"	int __acts = " << EA() << "[" << CS() << "]" << ";\n"
+				"	int __nacts = " << CAST("int") << " " << A() << "[__acts++];\n"
+				"	while ( __nacts-- > 0 ) {\n"
+				"		switch ( " << A() << "[__acts++] ) {\n";
+				EOF_ACTION_SWITCH() <<
+				"		}\n"
+				"	}\n";
+		}
 
-	/* The if guarding on the error state. */
-	if ( redFsm->errState != 0 )
-		out << "	}";
-
-	/* The if guarding on empty string. */
-	if ( hasEnd )
-		out << "	}\n";
-
-	/* The execute block. */
-	out << "	}\n";
-}
-
-void JavaTabCodeGen::writeEOF()
-{
-	if ( redFsm->anyEofActions() ) {
 		out <<
-			"	int _acts = " << EA() << "[" << CS() << "]" << ";\n"
-			"	int _nacts = " << CAST("int") << " " << A() << "[_acts++];\n"
-			"	while ( _nacts-- > 0 ) {\n"
-			"		switch ( " << A() << "[_acts++] ) {\n";
-			EOF_ACTION_SWITCH() <<
-			"		}\n"
 			"	}\n"
 			"\n";
 	}
+
+	out << "case " << _out << ":\n"; 
+
+	/* The switch and goto loop. */
+	out << "	}\n";
+	out << "	break; }\n";
+
+	/* The execute block. */
+	out << "	}\n";
 }
 
 std::ostream &JavaTabCodeGen::OPEN_ARRAY( string type, string name )
@@ -1041,54 +1150,62 @@ std::ostream &JavaTabCodeGen::OPEN_ARRAY( string type, string name )
 	item_count = 0;
 	div_count = 1;
 
-	out << 
-		"private static void init_" << name << "_0( " << type << "[] r )\n"
-		"{\n\t";
-
+	out <<  "private static " << type << "[] init_" << name << "_0()\n"
+		"{\n\t"
+		"return new " << type << " [] {\n\t";
 	return out;
 }
 
 std::ostream &JavaTabCodeGen::ARRAY_ITEM( string item, bool last )
 {
-	out << "r[" << item_count << "]=" << item << "; ";
+	item_count++;
 
-	item_count += 1;
+	out << setw(5) << setiosflags(ios::right) << item;
 	
 	if ( !last ) {
 		if ( item_count % SAIIC == 0 ) {
-			out << "\n}\n\n";
-			out << "private static void init_" << array_name << "_" << div_count << 
-					"( " << array_type << "[] r )\n{\n\t";
-			div_count += 1;
+			out << "\n\t};\n};\n"
+				"private static "<< array_type << "[] init_" << 
+				array_name << "_" << div_count << "()\n"
+				"{\n\t"
+				"return new " << array_type << " [] {\n\t";
+			div_count++;
+		} else if (item_count % IALL == 0) { 
+			out << ",\n\t";
+		} else {
+			out << ",";
 		}
-		else if ( item_count % IALL == 0 )
-			out << "\n\t";
 	}
-
 	return out;
 }
 
 std::ostream &JavaTabCodeGen::CLOSE_ARRAY()
 {
-	out << "\n}\n\n";
-
-	out << 
-		"private static " << array_type << "[] create_" << array_name << "( )\n"
-		"{\n"
-		"	" << array_type << "[] r = new " << array_type << "[" << item_count << "];\n";
-
-	for ( int i = 0; i < div_count; i++ )
-		out << "	init_" << array_name << "_" << i << "( r );\n";
-
-	out <<
-		"	return r;\n"
-		"}\n"
-		"\n";
-
-	out << 
-		"private static final " << array_type << " " << array_name << 
-				"[] = create_" << array_name << "();\n\n";
+	out << "\n\t};\n}\n\n";
 
+	if (item_count < SAIIC) {
+		out << "private static final " << array_type << " " << array_name << 
+			"[] = init_" << array_name << "_0();\n\n";
+	} else {
+		out << "private static final " << array_type << " [] combine_" << array_name
+			<< "() {\n\t"
+			<< array_type << " [] combined = new " << array_type << 
+			" [ " << item_count << " ];\n\t";
+		int block = 0;
+		int full_blocks = item_count / SAIIC;
+		for (;block < full_blocks; ++block) {
+			out << "System.arraycopy ( init_" << array_name << "_" << block << 
+				"(), 0, combined, " << SAIIC * block << ", " << SAIIC << " );\n\t";
+		}
+		if ( (item_count % SAIIC) > 0 ) {
+			out << "System.arraycopy ( init_" << array_name << "_" << block << 
+				"(), 0, combined, " << SAIIC * block << ", " << 
+				(item_count % SAIIC) << " );\n\t";
+		}
+		out << "return combined;\n}\n";
+		out << "private static final " << array_type << " [] " << array_name << 
+			" = combine_" << array_name << "();";
+	}
 	return out;
 }
 
@@ -1222,6 +1339,19 @@ string JavaTabCodeGen::PE()
 	return ret.str();
 }
 
+string JavaTabCodeGen::EOFV()
+{
+	ostringstream ret;
+	if ( eofExpr == 0 )
+		ret << "eof";
+	else {
+		ret << "(";
+		INLINE_LIST( ret, eofExpr, 0, false );
+		ret << ")";
+	}
+	return ret.str();
+}
+
 string JavaTabCodeGen::CS()
 {
 	ostringstream ret;
@@ -1279,7 +1409,7 @@ string JavaTabCodeGen::TOKSTART()
 {
 	ostringstream ret;
 	if ( tokstartExpr == 0 )
-		ret << ACCESS() + "tokstart";
+		ret << ACCESS() + "ts";
 	else {
 		ret << "(";
 		INLINE_LIST( ret, tokstartExpr, 0, false );
@@ -1292,7 +1422,7 @@ string JavaTabCodeGen::TOKEND()
 {
 	ostringstream ret;
 	if ( tokendExpr == 0 )
-		ret << ACCESS() + "tokend";
+		ret << ACCESS() + "te";
 	else {
 		ret << "(";
 		INLINE_LIST( ret, tokendExpr, 0, false );
@@ -1366,7 +1496,10 @@ void JavaTabCodeGen::LM_SWITCH( ostream &ret, InlineItem *item,
 
 	for ( InlineList::Iter lma = *item->children; lma.lte(); lma++ ) {
 		/* Write the case label, the action and the case break. */
-		ret << "	case " << lma->lmId << ":\n";
+		if ( lma->lmId < 0 )
+			ret << "	default:\n";
+		else
+			ret << "	case " << lma->lmId << ":\n";
 
 		/* Write the block and close it off. */
 		ret << "	{";
@@ -1375,9 +1508,8 @@ void JavaTabCodeGen::LM_SWITCH( ostream &ret, InlineItem *item,
 
 		ret << "	break;\n";
 	}
-	/* Default required for D code. */
+
 	ret << 
-		"	default: break;\n"
 		"	}\n"
 		"\t";
 }
