@@ -1,4 +1,5 @@
 /*
+ *  2007 Victor Hugo Borja <vic@rubyforge.org>
  *  Copyright 2001-2007 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
@@ -27,9 +28,46 @@
 
 #define PROGNAME "rlgen-ruby"
 
+/* Target implementation */
+enum RubyImplEnum
+{
+  MRI,
+  Rubinius
+};
+
+extern RubyImplEnum rubyImpl;
+
+/* Target output style. */
+enum CodeStyleEnum
+{
+	GenTables,
+	GenFTables,
+	GenFlat,
+	GenFFlat,
+	GenGoto,
+	GenFGoto,
+	GenIpGoto,
+	GenSplit
+
+};
+
+extern CodeStyleEnum codeStyle;
+
 extern int gblErrorCount;
 extern char machineMain[];
 
+/* Options. */
+extern int numSplitPartitions;
+extern bool noLineDirectives;
+
 std::ostream &error();
 
+/*
+ * Local Variables:
+ * mode: c++
+ * indent-tabs-mode: 1
+ * c-file-style: "bsd"
+ * End:
+ */
+
 #endif /* _RLGEN_RUBY_H */
